/*
 * Kenwood TS480SAT driver
 * originally based on Kenwood TS2000 driver
 *
 * a part of flrig
 *
 * Copyright 2009, Dave Freese, W1HKJ
 *
 */


#include "TS480SAT.h"
#include "support.h"

static const char TS480SATname_[] = "TS-480SAT";

static const char *TS480SATmodes_[] = {
		"LSB", "USB", "CW", "FM", "AM", "FSK", "CW-R", "FSK-R", NULL};
static const char TS480SAT_mode_chr[] =  { '1', '2', '3', '4', '5', '6', '7', '9' };
static const char TS480SAT_mode_type[] = { 'L', 'U', 'U', 'U', 'U', 'L', 'L', 'U' };

static const char *TS480SAT_widths[] = {
"50", "100", "250", "500", "1000", "1500", "2400", NULL};

RIG_TS480SAT::RIG_TS480SAT() {
// base class values
	name_ = TS480SATname_;
	modes_ = TS480SATmodes_;
	_mode_type = TS480SAT_mode_type;
	bandwidths_ = TS480SAT_widths;
	comm_baudrate = BR4800;
	stopbits = 2;
	comm_retries = 2;
	comm_wait = 5;
	comm_timeout = 50;
	comm_rtscts = true;
	comm_rtsplus = false;
	comm_dtrplus = false;
	comm_catptt = true;
	comm_rtsptt = false;
	comm_dtrptt = false;
	modeA = 1;
	bwA = 1;
	def_mode = 1;
	defbw_ = 1;
	deffreq_ = 14070000;

	has_noise_control =
	has_micgain_control =
	has_tune_control =
	has_preamp_control =
	has_notch_control =
	has_ifshift_control =
	has_swr_control = false;

	has_attenuator_control =
	has_mode_control =
	has_bandwidth_control =
	has_volume_control =
	has_power_control =
	has_ptt_control = true;
}

bool RIG_TS480SAT::sendTScommand(string cmd, int retnbr, bool loghex)
{
	int ret = sendCommand(cmd, retnbr, loghex);
	if (RigSerial.IsOpen()) {
//		LOG_INFO("%s", cmd.c_str());
//		if (retnbr)
//			LOG_INFO("%s", replybuff);
		return ret;
	}
	return 0;
}

long RIG_TS480SAT::get_vfoA ()
{
	cmd = "FA;";
	if (!sendTScommand(cmd, 14, false))
		return freqA;
	if (replystr.find("FA") != 0) {
		clearSerialPort();
		return freqA;
	}
	int f = 0;
	for (size_t n = 2; n < 13; n++)
		f = f*10 + replybuff[n] - '0';
	freqA = f;
	return freqA;
}

void RIG_TS480SAT::set_vfoA (long freq)
{
	freqA = freq;
	cmd = "FA00000000000;";
	for (int i = 12; i > 1; i--) {
		cmd[i] += freq % 10;
		freq /= 10;
	}
	sendTScommand(cmd, 0, false);
}

// SM cmd 0 ... 100 (rig values 0 ... 15)
int RIG_TS480SAT::get_smeter()
{
	cmd = "SM0;";
	if(!sendTScommand(cmd, 8, false))
		return 0;
	if (replystr.find("SM") != 0) {
		clearSerialPort();
		return 0;
	}
	replybuff[7] = 0;
	int mtr = atoi(&replybuff[3]);
	mtr = (mtr * 100) / 20;
	return mtr;
}

// RM cmd 0 ... 100 (rig values 0 ... 8)
int RIG_TS480SAT::get_swr()
{
	cmd = "RM;"; // select measurement '1' (swr) and read meter
	if (!sendTScommand(cmd, 24, false))
		return 0;
	if (replystr.find("RM") != 0) {
		clearSerialPort();
		return 0;
	}
	replybuff[7] = 0;
	int mtr = atoi(&replybuff[4]);
	mtr = (mtr * 100) / 10;
	return mtr;
}


// Tranceiver PTT on/off
void RIG_TS480SAT::set_PTT_control(int val)
{
	if (val)	cmd = "TX1;"; // DTS transmission using ANI input
	else	 	cmd = "RX;";
	sendTScommand(cmd, 4, false);
}

void RIG_TS480SAT::set_modeA(int val)
{
	modeA = val;
	cmd = "MD";
	cmd += TS480SAT_mode_chr[val];
	cmd += ';';
	sendTScommand(cmd, 4, false);
}

int RIG_TS480SAT::get_modeA()
{
	if (!sendTScommand("MD;", 4, false))
		return modeA;
	if (replystr.find("MD") != 0) {
		clearSerialPort();
		return modeA;
	}
	int md = replybuff[2];
	md = md - '1';
	if (md == 8) md = 7;
	modeA = md;
	return modeA;
}

int RIG_TS480SAT::get_modetype(int n)
{
	return _mode_type[n];
}

void RIG_TS480SAT::set_bwA(int val)
{
	bwA = val;
	cmd = "SH01"; // set center frequency to 1500
	sendTScommand(cmd, 0, false);
	cmd = "SL00";
	cmd[3] = '0' + val;
	sendTScommand(cmd, 0, false);
}

int RIG_TS480SAT::get_bwA()
{
	cmd = "SL;";
	if (sendTScommand(cmd, 5, false)) {
		bwA = replybuff[3] - '0';
		if (bwA < 0) bwA = 0;
		if (bwA > 6) bwA = 6;
	}
	return bwA;
}

void RIG_TS480SAT::set_volume_control(int val)
{
	cmd = "AG";
	char szval[5];
	snprintf(szval, sizeof(szval), "%04d", val * 255 / 100);
	cmd += szval;
	cmd += ';';
	sendTScommand(cmd, 0, false);
}

int RIG_TS480SAT::get_volume_control()
{
	cmd = "AG0";
	int val = 0;
	if (sendTScommand(cmd, 7, false)) {
		replybuff[6] = 0;
		val = atoi(&replybuff[3]);
		val = val * 100 / 255;
	}
	return val;
}

void RIG_TS480SAT::set_power_control(double val)
{
	cmd = "PC";
	char szval[4];
	if (modeA == 4 && val > 25) val = 25; // AM mode limitation
	snprintf(szval, sizeof(szval), "%03d", (int)val);
	cmd += szval;
	cmd += ';';
	sendTScommand(cmd, 0, false);
}

int RIG_TS480SAT::get_power_control()
{
	cmd = "PC;";
	int val = 5;
	if (sendTScommand(cmd, 6, false)) {
		replybuff[5] = 0;
		val = atoi(&replybuff[2]);
	}
	return val;
}

void RIG_TS480SAT::set_attenuator(int val)
{
	if (val)	cmd = "RA01";
	else		cmd = "RA00";
	sendTScommand(cmd, 0, false);
}

int RIG_TS480SAT::get_attenuator()
{
	cmd = "RA;";
	if (sendTScommand(cmd, 7, false)) {
		return replybuff[3] - '0';
	}
	return 0;
}

